"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var plugin_node_resolve_1 = __importDefault(require("@rollup/plugin-node-resolve"));
var rollup_plugin_svelte_1 = __importDefault(require("rollup-plugin-svelte"));
var plugin_virtual_1 = __importDefault(require("@rollup/plugin-virtual"));
var rollup_plugin_terser_1 = require("rollup-plugin-terser");
var preprocessReadme_1 = require("./preprocessReadme");
var fs_extra_1 = __importDefault(require("fs-extra"));
var path_1 = __importDefault(require("path"));
var crypto_1 = require("crypto");
var html_minifier_1 = __importDefault(require("html-minifier"));
var style_1 = require("./style");
function hashREADME() {
    try {
        var path_readme = path_1["default"].join(process.cwd(), "README.md");
        var readme = fs_extra_1["default"].readFileSync(path_readme);
        return "." + crypto_1.createHash("md5").update(readme).digest("hex").slice(0, 8);
    }
    catch (error) {
        console.log(error);
        process.exit(1);
    }
}
function getPackageJSON() {
    try {
        var path_pkg = path_1["default"].join(process.cwd(), "package.json");
        var pkg = JSON.parse(fs_extra_1["default"].readFileSync(path_pkg, "utf-8"));
        if (!pkg.name)
            throw Error("Package name is required as \"name\".");
        if (!pkg.svelte)
            throw Error("Svelte code entry is required as \"svelte\".");
        return {
            name: pkg.name,
            svelte: pkg.svelte,
            description: pkg.description,
            homepage: pkg.homepage
        };
    }
    catch (error) {
        console.log(error);
        process.exit(1);
    }
}
var custom_css = "\n  .token.language-javascript { color: #24292e; }\n  .token.language-javascript .function { color: #005cc5; }\n  .token.language-javascript .string { color: #032f62; }\n  .token.language-javascript .number { color: #005cc5; }\n  .token.language-javascript .keyword { color: #d73a49; }\n  .token.each { color: #d73a49; }\n  .token.punctuation { color: #24292e }\n  .token.tag { color: #22863a; }\n  .token.attr-name { color: #6f42c1; }\n  .token.attr-value { color: #032f62; }\n  .token.operator { color: #d73a49; }\n  .token.comment { color: #6a737d; }\n  .token.function { color: #6f42c1; }\n  .token.string { color: #032f62; }\n\n  .token.each .language-javascript:last-child { color: #e36209; }\n\n  .language-css { color: #032f62; }\n  .language-css .selector { color: #22863a; }\n  .language-css .property { color: #005cc5; }\n\n  .code-fence { padding: 24px 15px; border: 1px solid #eaecef; border-bottom: 0; }\n\n  main {\n    box-sizing: border-box;\n    max-width: 980px;\n    margin: 0 auto;\n    padding: 45px;\n  }\n\n  @media (max-width: 767px) {\n    main { padding: 15px; }\n  }\n";
function createConfig(opts) {
    var _a, _b;
    var DEV = process.env.ROLLUP_WATCH === "true";
    var minify = opts.minify === true || !DEV;
    var pkg = getPackageJSON();
    var hash = minify ? hashREADME() : "";
    var output_dir = opts.outDir || "dist";
    var svelte = {
        compilerOptions: {
            dev: DEV,
            immutable: true
        },
        extensions: [".svelte", ".md"],
        preprocess: [preprocessReadme_1.preprocessReadme(__assign(__assign({}, pkg), { prefixUrl: opts.prefixUrl }))]
    };
    if ((_a = opts.svelte) === null || _a === void 0 ? void 0 : _a.compilerOptions) {
        svelte.compilerOptions = __assign(__assign({}, svelte.compilerOptions), opts.svelte.compilerOptions);
    }
    if ((_b = opts.svelte) === null || _b === void 0 ? void 0 : _b.extensions) {
        svelte.extensions = __spreadArrays(svelte.extensions, opts.svelte.extensions);
    }
    console.log("[createConfig] Running in " + (DEV ? "development" : "production"));
    console.log("[createConfig] options:");
    console.group();
    console.log("minify:", minify);
    console.log("outDir:", output_dir);
    console.groupEnd();
    var template = "\n  <!DOCTYPE html>\n  <html lang=\"en\">\n    <head>\n      <meta charset=\"utf-8\" />\n      <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\" />\n      <meta name=\"description\" content=\"" + (pkg.description || pkg.name + " demo") + "\" />\n      <title>" + pkg.name + "</title>\n      <style>\n        " + (!opts.disableDefaultCSS ? style_1.css : "") + "\n        " + custom_css + "\n        " + (opts.style || "") + "\n      </style>\n    </head>\n    <body>\n      <noscript>You need to enable JavaScript to run this app.</noscript>\n      <script src=\"s" + hash + ".js\"></script>\n    </body>\n  </html>\n";
    if (minify)
        fs_extra_1["default"].removeSync(output_dir);
    fs_extra_1["default"].ensureFileSync(output_dir + "/index.html");
    fs_extra_1["default"].writeFileSync(output_dir + "/index.html", minify
        ? html_minifier_1["default"].minify(template, {
            collapseWhitespace: true,
            conservativeCollapse: true,
            minifyCSS: true,
            removeEmptyAttributes: true
        })
        : template);
    return {
        watch: { clearScreen: false },
        input: "entry",
        // @ts-ignore
        output: __assign({ format: "iife", name: "app", file: output_dir + "/s" + hash + ".js" }, (opts.output || {})),
        plugins: __spreadArrays([
            plugin_virtual_1["default"]({
                entry: "import App from \"./README.md\";\n                const app = new App({ target: document.body });\n                export default app;"
            }),
            rollup_plugin_svelte_1["default"](svelte),
            plugin_node_resolve_1["default"]()
        ], (opts.plugins || []), [
            minify && rollup_plugin_terser_1.terser(),
        ]).filter(Boolean)
    };
}
exports["default"] = createConfig;
